pub struct ComponentTypeCounter {
pub type_index: u32,
pub num_instances: u32,
}Fields
type_index: u32num_instances: u32Trait Implementations
Sourceimpl AsBrdbValue for ComponentTypeCounter
impl AsBrdbValue for ComponentTypeCounter
Sourcefn as_brdb_struct_prop_value(
&self,
schema: &BrdbSchema,
_struct_name: BrdbInterned,
prop_name: BrdbInterned,
) -> Result<&dyn AsBrdbValue, BrdbSchemaError>
fn as_brdb_struct_prop_value( &self, schema: &BrdbSchema, _struct_name: BrdbInterned, prop_name: BrdbInterned, ) -> Result<&dyn AsBrdbValue, BrdbSchemaError>
Read a specific struct property value from the schema.
fn as_brdb_bool(&self) -> Result<bool, BrdbSchemaError>
fn as_brdb_u8(&self) -> Result<u8, BrdbSchemaError>
fn as_brdb_u16(&self) -> Result<u16, BrdbSchemaError>
fn as_brdb_u32(&self) -> Result<u32, BrdbSchemaError>
fn as_brdb_u64(&self) -> Result<u64, BrdbSchemaError>
fn as_brdb_i8(&self) -> Result<i8, BrdbSchemaError>
fn as_brdb_i16(&self) -> Result<i16, BrdbSchemaError>
fn as_brdb_i32(&self) -> Result<i32, BrdbSchemaError>
fn as_brdb_i64(&self) -> Result<i64, BrdbSchemaError>
fn as_brdb_f32(&self) -> Result<f32, BrdbSchemaError>
fn as_brdb_f64(&self) -> Result<f64, BrdbSchemaError>
fn as_brdb_str(&self) -> Result<&str, BrdbSchemaError>
fn as_brdb_asset( &self, _schema: &BrdbSchema, _ty: &str, ) -> Result<Option<usize>, BrdbSchemaError>
fn as_brdb_enum( &self, _schema: &BrdbSchema, _def: &BrdbSchemaEnum, ) -> Result<i32, BrdbSchemaError>
fn as_brdb_wire_variant(&self) -> Result<WireVariant, BrdbSchemaError>
Sourcefn as_brdb_struct_prop_array(
&self,
_schema: &BrdbSchema,
_struct_name: BrdbInterned,
_prop_name: BrdbInterned,
) -> Result<BrdbArrayIter<'_>, BrdbSchemaError>
fn as_brdb_struct_prop_array( &self, _schema: &BrdbSchema, _struct_name: BrdbInterned, _prop_name: BrdbInterned, ) -> Result<BrdbArrayIter<'_>, BrdbSchemaError>
Get the the number of entries in a struct property.
Sourcefn as_brdb_struct_prop_map(
&self,
_schema: &BrdbSchema,
_struct_name: BrdbInterned,
_prop_name: BrdbInterned,
) -> Result<BrdbMapIter<'_>, BrdbSchemaError>
fn as_brdb_struct_prop_map( &self, _schema: &BrdbSchema, _struct_name: BrdbInterned, _prop_name: BrdbInterned, ) -> Result<BrdbMapIter<'_>, BrdbSchemaError>
Get the the number of entries in a struct property.
Sourceimpl Clone for ComponentTypeCounter
impl Clone for ComponentTypeCounter
Sourcefn clone(&self) -> ComponentTypeCounter
fn clone(&self) -> ComponentTypeCounter
Returns a duplicate of the value. Read more
1.0.0 · Sourcefn clone_from(&mut self, source: &Self)
fn clone_from(&mut self, source: &Self)
Performs copy-assignment from
source. Read moreSourceimpl Debug for ComponentTypeCounter
impl Debug for ComponentTypeCounter
Sourceimpl Default for ComponentTypeCounter
impl Default for ComponentTypeCounter
Sourcefn default() -> ComponentTypeCounter
fn default() -> ComponentTypeCounter
Returns the “default value” for a type. Read more
Sourceimpl Ord for ComponentTypeCounter
impl Ord for ComponentTypeCounter
Sourcefn cmp(&self, other: &ComponentTypeCounter) -> Ordering
fn cmp(&self, other: &ComponentTypeCounter) -> Ordering
1.21.0 · Sourcefn max(self, other: Self) -> Selfwhere
Self: Sized,
fn max(self, other: Self) -> Selfwhere
Self: Sized,
Compares and returns the maximum of two values. Read more
Sourceimpl PartialEq for ComponentTypeCounter
impl PartialEq for ComponentTypeCounter
Sourceimpl PartialOrd for ComponentTypeCounter
impl PartialOrd for ComponentTypeCounter
Sourceimpl TryFrom<&BrdbValue> for ComponentTypeCounter
impl TryFrom<&BrdbValue> for ComponentTypeCounter
impl Copy for ComponentTypeCounter
impl Eq for ComponentTypeCounter
impl StructuralPartialEq for ComponentTypeCounter